import { dirname, join } from "node:path";
import type * as vite from "vite";
import { memoize } from "lodash";
import { type Compile, type StyleMap, pkgName, toCSSText } from "./shared";

const ROLLUP_PREFIX = "\0tailwind:";

export const vitePlugin = (styleMap: StyleMap, compile: Compile): vite.Plugin => ({
  name: "tailwind",
  resolveId(id, importer) {
    if (id === `${pkgName}/base`) {
      return {
        id: ROLLUP_PREFIX + "directive:base",
        moduleSideEffects: true,
      };
    }

    if (id.startsWith("tailwind:")) {
      const resolved = join(dirname(importer!), id.slice("tailwind:".length));
      if (styleMap.has(resolved)) {
        return {
          id: ROLLUP_PREFIX + resolved,
          moduleSideEffects: true,
        };
      }
    }
  },
  async load(id: string) {
    if (id.startsWith(ROLLUP_PREFIX)) {
      const resolved = id.slice(ROLLUP_PREFIX.length);
      if (resolved === "directive:base") {
        return (await compile("@tailwind base")).css;
      }

      if (styleMap.has(resolved)) {
        const result = await compile(toCSSText(styleMap.get(resolved)!));
        return result.css;
      }
    }
  },
});
